#!/usr/local/bin python3
#  coding: utf-8

import pyglet
import math
import numpy as np
import random


import shapefile  # pyshp package

import ctypes
import csv
import timeit
from codecs import open

# import platform
# import bisect
# import time

#################################################
# Use pyglet for our graphic rendering          #
#                                               #
# https://pythonhosted.org/pyglet/api/          #
# https://bitbucket.org/pyglet/pyglet/wiki/Home #
#                                               #
from pyglet.window import key                   #
from pyglet.window import mouse                 #
# pyglet is above opengl and exposes opengl     #
# for low level rendering                       #
from pyglet.gl import gl                        #
from pyglet.gl import GLubyte
# from pyglet.gl import glu_info
# from pyglet.gl import glu
#
#                                               #
#################################################


#################################################
# Initialize windows and other system stuff     #
window = pyglet.window.Window(1280, 1024,       #
                              resizable=True)   #
window.set_location(0, 0)                       #
# window.set_fullscreen(True)                   #
window.set_caption('StripMap')                  #
# window.set_exclusive_mouse(True)              #
#################################################


# taken on internet
# :https://gamedev.stackexchange.com/questions/59432/pyglet-text-label-draw-to-fbo
def SetScreen(self, left, top, right, bottom):
    gl.glMatrixMode(gl.GL_MODELVIEW)
    gl.glLoadIdentity()
    gl.glMatrixMode(gl.GL_PROJECTION)
    gl.glLoadIdentity()
    gl.glOrtho(left, right, bottom, top, -1.0, 1.0)
    gl.glMatrixMode(gl.GL_MODELVIEW)


# taken on internet
# :https://gamedev.stackexchange.com/questions/59432/pyglet-text-label-draw-to-fbo
def SetRenderTarget(texture=None):
    global FBO
    if not texture:
        # use screen when no texture was specified
        gl.glBindFramebufferEXT(gl.GL_FRAMEBUFFER_EXT, 0)
        gl.glViewport(0, 0, gl.SCREEN_WIDTH, gl.SCREEN_HEIGHT)
        gl.SetScreen(0, 0, gl.SCREEN_WIDTH, gl.SCREEN_HEIGHT)
    else:
        # create FBO object once
        if not FBO:
            gl.FBO = gl.GLuint()
            gl.glGenFramebuffersEXT(1, ctypes.byref(FBO))

        # bind the frame buffer
        gl.glBindFramebufferEXT(gl.GL_FRAMEBUFFER_EXT, FBO)

        # check that the texture has a depth render-buffer (not strictly
        # neccessary)
        try:
            _t = texture.DepthRB
        except:
            texture.DepthRB = gl.GLuint()
            gl.glGenRenderbuffersEXT(1, ctypes.byref(texture.DepthRB))
            gl.glBindRenderbufferEXT(gl.GL_RENDERBUFFER_EXT, texture.DepthRB)
            gl.glRenderbufferStorageEXT(gl.GL_RENDERBUFFER_EXT, gl.GL_DEPTH_COMPONENT16,
                                        texture.width, texture.height)

        # bind texture and depth buffer to FBO
        gl.glBindTexture(texture.target, texture.id)
        gl.glFramebufferTexture2DEXT(gl.GL_FRAMEBUFFER_EXT, gl.GL_COLOR_ATTACHMENT0_EXT,
                                     texture.target, texture.id, 0)
        gl.glFramebufferRenderbufferEXT(gl.GL_FRAMEBUFFER_EXT, gl.GL_DEPTH_ATTACHMENT_EXT,
                                        gl.GL_RENDERBUFFER_EXT, texture.DepthRB)

        # clear FBO and set some GL defaults
        gl.glEnable(gl.GL_BLEND)
        gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
        gl.glEnable(gl.GL_DEPTH_TEST)
        gl.glDepthFunc(gl.GL_LEQUAL)
        gl.glClearDepth(1.0)
        gl.glClearColor(0.5, 0.0, 0.5, 0.0)  # set clear color yourself!
        gl.glViewport(0, 0, texture.width, texture.height)
        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
        gl.SetScreen(0, 0, texture.width, texture.height)

        # simple error checking
        status = gl.glCheckFramebufferStatusEXT(gl.GL_FRAMEBUFFER_EXT)
        assert status == gl.GL_FRAMEBUFFER_COMPLETE_EXT


# taken on internet
# :https://gamedev.stackexchange.com/questions/59432/pyglet-text-label-draw-to-fbo
def CreateTexture(size=512, fmt=gl.GL_RGBA, data=None):
    if type(size) == int:
        w = size
        h = size
    else:
        w = int(size[0])
        h = int(size[1])

    tid = gl.GLuint()
    target = gl.GL_TEXTURE_2D

    gl.glGenTextures(1, byref(tid))
    gl.glBindTexture(target, tid.value)
    gl.glTexParameteri(target, gl.GL_TEXTURE_MIN_FILTER, gl.GL_LINEAR)

    if not data:
        blank = (GLubyte * (w * h * 4))()
        data = blank

    gl.glTexImage2D(target, 0, fmt, w, h, 0, gl.GL_RGBA,
                    gl.GL_UNSIGNED_BYTE, data)

    return pyglet.image.Texture(w, h, target, tid.value)


# logos ... to keep peace in the research world
logo4 = pyglet.image.load('images/limsi-transparent.png')
logo2 = pyglet.image.load('images/aviz.png')
logo3 = pyglet.image.load('images/CNRS.png')
logo1 = pyglet.image.load('images/logo-psud-saclay.jpg')
image1 = pyglet.image.load('images/Stratum.jpg')

# for the progressbar
maxpb = 100
pb = 0

# for the strip
angle = 20
x0 = 100
y0 = 60
graphHeight = 64
graphWidth = math.sqrt(window.width*window.width+window.height*window.height)

# for the double ZUI
zoom = 8.0  # map zoom
depzoom = 1.0  # individual departement zoom

# gloab keyboard modifier to access it on wheel events
glomodifier = 0

# data ! 11 candidates+sum along a strip/graph of 100 values
vals = [[0.0 for i in range(100)] for j in range(12)]
alldeps = {}
allcolors = ((0, 0, 128), (64, 64, 255),
             (255, 128, 0), (128, 64, 64), (255, 0, 0))

# vertices in the strip
vertex_graphs = pyglet.graphics.Batch()
NBMAXSTRIPS = 20
NBSTRIPS = NBMAXSTRIPS
NBCANDS = 5
vl = [None] * (NBCANDS * NBSTRIPS)

# default messages in the console and in the window
print(' display ' + repr(window.width) + 'x' + repr(window.height))
msg = pyglet.text.Label('Initialisation',
                        font_name='Times New Roman',
                        font_size=36,
                        x=0, y=window.height // 2,
                        anchor_x='center', anchor_y='center')

docud = pyglet.text.document.FormattedDocument("???")
docud.set_style(0, len(docud.text), dict(
    color=(255, 255, 255, 255), background_color=(0, 0, 0, 156), bold=True))
detcom = pyglet.text.layout.TextLayout(docud, window.width, 50, multiline=True)
detcom.x = 2.35
detcom.y = 48.8
detcom.anchor_x = 'left'
detcom.anchor_y = 'bottom'


# communes stuff. where the data is
communes = {}
comdetail = []


docul = pyglet.text.document.FormattedDocument("Press X to reset map to full France.\n\
   Press S to switch between 1 strip and multiple strips\nMouse Wheel+Drag'n Drop to zoom and pan\n\
   ctrl+wheel to rotate strips\ncmd+wheel to rotate strip(s)\n\n\n")
docul.set_style(0, len(docul.text), dict(color=(96, 255, 32, 255),
                                         background_color=(0, 128, 255, 156), bold=True))
legend = pyglet.text.layout.TextLayout(docul, window.width, 50, multiline=True)
legend.x = -window.width // 2 + 8
legend.y = -window.height // 2 + 20
legend.anchor_x = 'left'
legend.anchor_y = 'bottom'


# *******************************************************************************
#
#  Play with image processing
#
#  Discussion:
#
#    load the data. first list of communes with insee id and lon/lat
#    then french poll results with insee id in it too.
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license.
#
#  Modified:
#
#    3 July 2017
#
#  Author:
#
#    Fred Vernier
#
#  Parameters:
#
#
def loadData():
    global pb, maxpb, msg, window, communes, NBCANDS

    start = timeit.default_timer()

    maxpb = 73
    pb = 0
    communes = {}
    msg = pyglet.text.Label('Loading communes data (general infos)',
                            font_name='Times New Roman',
                            font_size=36,
                            x=0, y=0,
                            anchor_x='center', anchor_y='center')

    # load city data
    # http://sql.sh/736-base-donnees-villes-francaises
    with open('data/villes_france.csv', 'r', encoding='utf-8') as csvfile:
        csvreader = csv.reader(csvfile, delimiter=',', quotechar='"')
        for j, row in enumerate(csvreader):
            # if j==0 or row[2]=="pressiat":
            #   print("\n"+repr(int(row[10].replace('2A', '200').replace('2B', '210')))+": "+', '.join(row)+" === "+repr(len(row)))
            if j > -1 and row[1] in alldeps:
                lon = float(row[19])
                lat = float(row[20])
                insee = int(row[10].replace('2A', '200').replace('2B', '210'))
                dep = alldeps[row[1]]
                dep['ins'] += insee,
                dep['vert'] += (lon, lat)
                dep['col'] += (255, 0, 255)
                communes[insee] = [row[0], row[1], row[2], lat, lon]
            if j % 500 == 0:
                pb = j / 500
                # update the window to update the progressbar while loading
                window.switch_to()
                window.dispatch_events()
                update(-1)
                window.flip()
        for k, dep in alldeps.items():
            batch3 = pyglet.graphics.Batch()
            # print(dep['name']+" => "+repr(len(dep['vert'])))

            if len(dep['vert']) > 0:
                batch3.add(len(dep['vert']) // 2,
                           gl.GL_POINTS, None,
                           ('v2f/static', dep['vert']),
                           ('c3B/static', dep['col'])
                           )
                dep['compoints'] = batch3

    # reset progressbar
    pb = 0
    maxpb = 71
    msg = pyglet.text.Label('Loading communes data (poll results)',
                            font_name='Times New Roman',
                            font_size=36,
                            x=0, y=0,
                            anchor_x='center', anchor_y='center')

    # load poll data
    with open('data/Presidentielle_2017_T1.csv', 'r', encoding='utf-8') as csvfile:
        csvreader = csv.reader(csvfile, delimiter=',', quotechar='"')
        for j, row in enumerate(csvreader):
            # if j==0:
            #   print(', '.join(row))
            if (j > 0 and row[0] != "ZA" and row[0] != "ZB" and row[0] != "ZC" and row[0] != "ZD" and row[0] != "ZM" and
                    row[0] != "ZN" and row[0] != "ZP" and row[0] != "ZZ" and row[0] != "ZS" and row[0] != "ZW" and row[0] != "ZX"):
                insee = int(row[0].replace('2A', '200').replace(
                    '2B', '210')) * 1000 + int(row[2])
                if insee in communes:
                    communes[insee].extend([0.0] * (NBCANDS + 1))
                    communes[insee][5] = float(row[15])
                    communes[insee][6] = 0.0
                    communes[insee][7] = 0.0
                    communes[insee][8] = 0.0
                    communes[insee][9] = 0.0
                    communes[insee][10] = 0.0
                    # if insee==1284:
                    #   print(', '.join(row))
                    for i in range(18, 18 + 7 * 11, 7):
                        if row[i + 2] == "LE PEN":
                            communes[insee][6] = float(row[i + 4])
                        if row[i + 2] == "FILLON":
                            communes[insee][7] = float(row[i + 4])
                        if row[i + 2] == "MACRON":
                            communes[insee][8] = float(row[i + 4])
                        if row[i + 2] == "HAMON":
                            communes[insee][9] = float(row[i + 4])
                        if row[i + 2] == "MÉLENCHON":
                            communes[insee][10] = float(row[i + 4])

                    '''
                    for i in range(17, 17+2*6, 6):
                      if row[i+2]=="LE PEN"   : communes[insee][28] = float(row[i+4])
                      if row[i+2]=="MACRON"   : communes[insee][29] = float(row[i+4])
                    '''

                else:
                    print("ERROR communes: " + repr(int(row[0]) * 1000 + int(
                        row[2])) + " : " + row[3] + " in " + row[1])
            if j % 500 == 0:
                pb = j / 500
                # update the window to update the progressbar
                window.switch_to()
                window.dispatch_events()
                update(-1)
                window.flip()

    # no more progress bar
    pb = -1

    stop = timeit.default_timer()
    print("loadData " + repr(stop - start))


# *****************************************************************************80
#
#  Play with image processing. NOT USED HERE
#
#  Discussion:
#
#    DO NOT acess window.width or image1.width inside the loop...
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license.
#
#  Modified:
#
#    3 July 2017
#
#  Author:
#
#    Fred Vernier
#
#  Parameters:
#
#
def computeImages():
    global pb, maxpb, msg, window, image1
    msg = pyglet.text.Label('Processing images',
                            font_name='Times New Roman',
                            font_size=36,
                            x=0,
                            y=0,
                            anchor_x='center',
                            anchor_y='center')

    ww = image1.width
    hh = image1.height

    # pixels2 = [0] * ww * hh *3;
    rawimage1 = image1.get_image_data()
    pixels1 = rawimage1.get_data(rawimage1.format, rawimage1.pitch)
    print(' image ' + repr(rawimage1.width) + 'x' +
          repr(rawimage1.height) + " => " + repr(rawimage1.pitch))
    # event_loop = pyglet.app.EventLoop()
    maxpb = rawimage1.height
    pb = 0

    for j in range(0, hh):
        casey = j // 40

        # update the window to update the progressbar
        if (j % 10) == 0:
            pb = pb + 10
            window.switch_to()
            window.dispatch_events()
            update(-1)
            window.flip()

        for i in range(0, ww):
            casex = (i + (casey % 2) * 20) // 40
            r1 = 65 - math.hypot(ww / 2 - (+00 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (+00 + casey * 40 + 20)) / 12
            r2 = 65 - math.hypot(ww / 2 - (+40 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (+00 + casey * 40 + 20)) / 12
            r3 = 65 - math.hypot(ww / 2 - (-40 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (+00 + casey * 40 + 20)) / 12
            r4 = 65 - math.hypot(ww / 2 - (-20 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (-40 + casey * 40 + 20)) / 12
            r5 = 65 - math.hypot(ww / 2 - (-20 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (+40 + casey * 40 + 20)) / 12
            r6 = 65 - math.hypot(ww / 2 - (+20 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (-40 + casey * 40 + 20)) / 12
            r7 = 65 - math.hypot(ww / 2 - (+20 + casex * 40 - 20 - (casey % 2) * 20), hh / 2 - (+40 + casey * 40 + 20)) / 12

            d1 = math.hypot(+00 + casex * 40 + 20 - (casey % 2) * 20 - i,  00 + casey * 40 + 20 - j) * 255 / (r1 + 7)
            d2 = math.hypot(+40 + casex * 40 + 20 - (casey % 2) * 20 - i,  00 + casey * 40 + 20 - j) * 255 / (r2 + 7)
            d3 = math.hypot(-40 + casex * 40 + 20 - (casey % 2) * 20 - i,  00 + casey * 40 + 20 - j) * 255 / (r3 + 7)
            d4 = math.hypot(-20 + casex * 40 + 20 - (casey % 2) * 20 - i, -40 + casey * 40 + 20 - j) * 255 / (r4 + 7)
            d5 = math.hypot(-20 + casex * 40 + 20 - (casey % 2) * 20 - i,  40 + casey * 40 + 20 - j) * 255 / (r5 + 7)
            d6 = math.hypot(+20 + casex * 40 + 20 - (casey % 2) * 20 - i, -40 + casey * 40 + 20 - j) * 255 / (r6 + 7)
            d7 = math.hypot(+20 + casex * 40 + 20 - (casey % 2) * 20 - i,  40 + casey * 40 + 20 - j) * 255 / (r7 + 7)

            d = 255 - min(min(min(round(d1), round(d2)), min(round(d3), round(d4))), min(min(round(d5), round(d6)), round(d7)))
            d = max(0, min(255, d))

            pixels1[i * 4 + j * rawimage1.pitch + 0] = (pixels1[i * 4 + j * rawimage1.pitch + 0] * d) // 255
            pixels1[i * 4 + j * rawimage1.pitch + 1] = (pixels1[i * 4 + j * rawimage1.pitch + 1] * d) // 255
            pixels1[i * 4 + j * rawimage1.pitch + 2] = (pixels1[i * 4 + j * rawimage1.pitch + 2] * d) // 255

    rawimage1.set_data(rawimage1.format, rawimage1.pitch, pixels1)


# *****************************************************************************80
#
# Make all the pre computation to draw the graphs later on
#
#  Discussion:
#
#    compute the triangles from polygons of french departements
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license.
#
#  Modified:
#
#    31 May 2017
#
#  Author:
#
#    Fred Vernier
#
#  Parameters:
#
#
def computePolygons():
    global triangles, x, y, pb, maxpb, msg, minlon, maxlon, minlat, maxlat, zoom, mmx, mmy

    start = timeit.default_timer()

    msg = pyglet.text.Label('Loading French Departements (shapes)',
                            font_name='Times New Roman',
                            font_size=36,
                            x=0, y=0,
                            anchor_x='center', anchor_y='center')

    # because there are little chance borders of France move ... exept if
    # french invade Germany again :-)
    minlon = -4.79619469819
    minlat = 41.3654098704
    maxlon = 9.560416
    maxlat = 51.0931719524

    zoom   = window.height / (maxlat - minlat)
    mmx    = (minlon + maxlon) / 2
    mmy    = (minlat + maxlat) / 2

    print('lon : ' + repr(minlon) + '...' + repr(maxlon))
    print('lat : ' + repr(minlat) + '...' + repr(maxlat))

    zoom = 1200
    mmx  = 2.3572
    mmy  = 48.8533

    # from https://www.data.gouv.fr/fr/datasets/contours-des-departements-francais-issus-d-openstreetmap/
    # https://www.data.gouv.fr/fr/datasets/decoupage-administratif-communal-francais-issu-d-openstreetmap/
    myshp = open(
        "departements-20140306-100m-shp/departements-20140306-100m.shp", "rb")
    mydbf = open(
        "departements-20140306-100m-shp/departements-20140306-100m.dbf", "rb")
    sf = shapefile.Reader(shp=myshp, dbf=mydbf)

    # reset progress bar
    pb = 0
    maxpb = len(sf.shapes())

    for s in list(sf.iterShapeRecords()):
        pb = pb + 1

        medlon = (s.shape.bbox[0] + s.shape.bbox[2]) / 2
        medlat = (s.shape.bbox[1] + s.shape.bbox[3]) / 2

        if isinstance(s.record[1], bytes):
            txt = s.record[1].decode('unicode_escape')
        else:
            txt = s.record[1]

        # create the structure for the french departement
        dep = {'num': s.record[0], 'nbpts': len(s.shape.points),
               'cenx': medlon,      'ceny': medlat,
               # poly is updated later
               'poly': None,        'name': s.record[1],
               'pieces': [],          'bbox': s.shape.bbox,
               'vert': (),           'col': (),
               'ins': (),
               'name': txt,         'label': pyglet.text.Label(txt,
                                                               font_name='Arial',
                                                               font_size=10,
                                                               color=(
                                                                   0, 0, 0, 255),
                                                               x=medlon,  y=medlat,
                                                               anchor_x='center', anchor_y='center')}

        # print a small survey
        # print (repr(s.record[0])+' = '+txt+' => '+repr(len(s.shape.points))+' pts '+repr(medlon))

        start  = 0
        # j = len(alldeps)
        batch1 = pyglet.graphics.Batch()
        batch2 = pyglet.graphics.Batch()

        s.shape.parts.remove(0)
        s.shape.parts.append(len(s.shape.points))
        for k, end in enumerate(s.shape.parts):
            vertices = ()
            vertices2 = ()
            colors = ()
            colors2 = ()
            triangles = []
            angles = []
            if start >= end:
                continue

            angles.append({'index': 0, 'angle': 0.000, 'distance': 0,
                           'x': 0, 'y': 0, 'next': 1, 'prev': end - start - 2})
            # if start==930: print('range '+repr(start)+'...'+repr(end-1))
            # to store previous values of x and y in the loop
            px = py = ppx = ppy = 0
            for i in range(start, end - 1):
                lon = s.shape.points[i][0]
                lat = s.shape.points[i][1]
                y = lat  # (lat-minlat)*(window.height)/(maxlat-minlat)-ceny
                # window.width/2+math.sin(math.pi/4)*(lon-midlon)*(window.height)/(maxlat-minlat)-cenx
                x = lon

                if i > start:
                    dd = math.sqrt((x - px) * (x - px) + (y - py) * (y - py))
                    vertices2 += (px, py)
                    vertices2 += (x, y)
                    colors2 += (000, 000, 000)
                    colors2 += (000, 000, 000)
                    if i > start + 1:
                        xx = (x - px) * (ppx - px) + (y - py) * (ppy - py)
                        yy = (x - px) * (ppy - py) - (y - py) * (ppx - px)
                        aa = math.atan2(yy, xx)
                        if aa < 0:
                            dd = math.sqrt((ppx - x) * (ppx - x) + (ppy - y) * (ppy - y))
                        else:
                            dd = math.inf
                        # if start==930: print('  add '+repr(i-start-1))
                        angles.append({'index': i - start - 1, 'angle': aa, 'distance': dd,
                                       'x': px, 'y': py, 'next': i - start, 'prev': i - start - 2})
                    ppx = px
                    ppy = py
                px = x
                py = y
                vertices += (x, y)
                # rr = random.randrange(0, 55)
                # if s.record[0]=='26':
                colors += ((pb % 11) * 25, 127 + (pb % 8) * 16, 128 + (pb % 14) * 5)
                # else:
                #  colors += ((k%2)*255, 200+rr, (k%3)*127)

                # if i>start+1 and dep['num']!='26':
                #  triangles.extend([0, i-1-start, i-start])

            vertices2 += (vertices2[len(vertices2) - 2],
                          vertices2[len(vertices2) - 1])
            vertices2 += (vertices2[0], vertices2[1])

            colors2 += (000, 000, 000)
            colors2 += (000, 000, 000)

            xx = (vertices[2] - vertices[0]) * (vertices[len(vertices) - 2] - vertices[0]) + (
                  vertices[3] - vertices[1]) * (vertices[len(vertices) - 1] - vertices[1])
            yy = (vertices[2] - vertices[0]) * (vertices[len(vertices) - 1] - vertices[1]) - (
                  vertices[3] - vertices[1]) * (vertices[len(vertices) - 2] - vertices[0])
            aa = math.atan2(yy, xx)
            if aa < 0:
                dd = math.sqrt((vertices[len(vertices) - 2] - vertices[2]) * (vertices[len(vertices) - 2] - vertices[2]) +
                               (vertices[len(vertices) - 1] - vertices[3]) * (vertices[len(vertices) - 1] - vertices[3]))
            else:
                dd = math.inf
            angles[0]['angle'] = aa
            angles[0]['distance'] = dd
            angles[0]['x'] = vertices[0]
            angles[0]['y'] = vertices[1]

            xx = (vertices[0] - vertices[len(vertices) - 2]) * (vertices[len(vertices) - 4] - vertices[len(vertices) - 2]) + (
                  vertices[1] - vertices[len(vertices) - 1]) * (vertices[len(vertices) - 3] - vertices[len(vertices) - 1])
            yy = (vertices[0] - vertices[len(vertices) - 2]) * (vertices[len(vertices) - 3] - vertices[len(vertices) - 1]) - (
                  vertices[1] - vertices[len(vertices) - 1]) * (vertices[len(vertices) - 4] - vertices[len(vertices) - 2])
            aa = math.atan2(yy, xx)
            if aa < 0:
                dd = math.sqrt((vertices[len(vertices) - 4] - vertices[0]) * (vertices[len(vertices) - 4] - vertices[0]) +
                               (vertices[len(vertices) - 3] - vertices[1]) * (vertices[len(vertices) - 3] - vertices[1]))
            else:
                dd = math.inf

            # if start==930: print('++  '+repr(end-start-2))
            angles.append({'index': end - start - 2, 'angle': aa, 'distance': dd,
                           'x': vertices[len(vertices) - 2], 'y': vertices[len(vertices) - 1], 'next': 0, 'prev': end - start - 3})

            sordistances = sorted(angles, key=lambda a: a[
                                  'distance'], reverse=False)
            cpt = 3000

            while len(sordistances) > 3 and cpt > 0:
                cpt = cpt - 1
                a = sordistances[0]
                if a['angle'] < 0 and a['index'] >= 0:
                    prev = a['prev']
                    next = a['next']
                    index = a['index']
                    # print('make tri '+repr(prev)+' '+repr(index)+' '+repr(next))

                    triangles.extend([prev, index, next])
                    angles[prev]['next'] = next
                    angles[next]['prev'] = prev
                    pprev = angles[prev]['prev']
                    x1 = angles[pprev]['x']
                    y1 = angles[pprev]['y']
                    x2 = angles[prev]['x']
                    y2 = angles[prev]['y']
                    x3 = angles[next]['x']
                    y3 = angles[next]['y']
                    xx = (x3 - x2) * (x1 - x2) + (y3 - y2) * (y1 - y2)
                    yy = (x3 - x2) * (y1 - y2) - (y3 - y2) * (x1 - x2)
                    aa = math.atan2(yy, xx)
                    if aa < 0:
                        dd = math.sqrt((x3 - x1) * (x3 - x1) + (y3 - y1) * (y3 - y1))
                    else:
                        dd = math.inf
                    angles[prev]['angle'] = aa
                    angles[prev]['distance'] = dd
                    sordistances.remove(a)

                    sordistances.remove(angles[prev])
                    le = 0
                    ri = len(sordistances) - 1
                    while le < ri - 1:
                        m = (le + ri) // 2
                        if sordistances[m]['distance'] >= dd:
                            ri = m
                        else:
                            le = m
                    sordistances.insert(ri, angles[prev])

                    a['angle'] = 50
                    a['distance'] = math.inf
                    a['index'] = -1

                    nnext = angles[next]['next']
                    x1 = angles[prev]['x']
                    y1 = angles[prev]['y']
                    x2 = angles[next]['x']
                    y2 = angles[next]['y']
                    x3 = angles[nnext]['x']
                    y3 = angles[nnext]['y']
                    xx = (x3 - x2) * (x1 - x2) + (y3 - y2) * (y1 - y2)
                    yy = (x3 - x2) * (y1 - y2) - (y3 - y2) * (x1 - x2)
                    aa = math.atan2(yy, xx)
                    if aa < 0:
                        dd = math.sqrt((x3 - x1) * (x3 - x1) + (y3 - y1) * (y3 - y1))
                    else:
                        dd = math.inf
                    angles[next]['angle'] = aa
                    angles[next]['distance'] = dd

                    sordistances.remove(angles[next])
                    le = 0
                    ri = len(sordistances) - 1
                    while le < ri - 1:
                        m = (le + ri) // 2
                        if sordistances[m]['distance'] >= dd:
                            ri = m
                        else:
                            le = m
                    sordistances.insert(ri, angles[next])

            if len(sordistances) == 3:
                triangles.extend([sordistances[0]['index'], sordistances[1]['index'], sordistances[2]['index']])
            else:
                # print(repr(start)+'...'+repr(end-1))
                # for sd in sordistances:
                #  print(sd)
                pass

            start = end

            batch1.add_indexed(len(vertices) // 2, gl.GL_TRIANGLES, None,
                                             triangles,
                                             ('v2f/static', vertices),
                                             ('c3B/static', colors))
            batch2.add(len(vertices2) // 2, gl.GL_LINES, None,
                                     ('v2f/static', vertices2),
                                     ('c3B/static', colors2))

            # for picking later on
            dep['pieces'].append(vertices)

        # for drawing later on
        dep['poly'] = batch1
        dep['border'] = batch2
        alldeps[dep["num"]] = dep

        # update the window to update the progressbar
        window.switch_to()
        window.dispatch_events()
        update(-1)
        window.flip()

    # no more progress bar
    pb = -1
    stop = timeit.default_timer()
    print("computePolygons " + repr(stop - start))


# *************************************************************************
#
# Make all the pre computation to draw the graphs later on
#
#  Discussion:
#    compute the position of all communes in the strips, aggragate them is
#    rotated rectangles along the strips, then normalize every score inside
#    rectangle. THere is then a "empty rect" filling pass and a smoothong pass
#    finally, triangles are computed to make the stacked graphs with the data
#
#  Licensing:
#    This code is distributed under the GNU LGPL license.
#
#  Modified:
#    31 May 2017
#
#  Author:
#    Fred Vernier
#
#  Parameters:
#
#
def computeGraphs():
    global vertex_graphs, x0, y0, angle, graphWidth, graphHeight, vals, mmx, mmy, zoom, NBSTRIPS, NBCANDS, vl
    # start = timeit.default_timer()

    vals = [[[0.0 for i in range(100)] for j in range(12)]
            for l in range(NBSTRIPS)]
    gl.glPushMatrix()
    gl.glTranslatef(graphWidth / 2, graphHeight / 2, 0)
    gl.glRotatef(-angle, 0, 0, 1)
    if NBSTRIPS == 1:
        gl.glTranslatef(-x0, -y0, 0)
    gl.glScalef(zoom * math.sin(math.pi / 4), zoom, 1)
    gl.glTranslatef(-mmx, -mmy, 0)
    a = (ctypes.c_float * 16)()
    gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
    a = np.array(a)
    a = np.reshape(a, (4, 4))
    a = a.T
    gl.glPopMatrix()

    # print(len(communes))
    # stop = timeit.default_timer()
    # print ("computeGraphs1 "+repr(stop - start))

    b = np.array([0, 0, 0, 1])
    for k, com in communes.items():
        # if com[1] in alldeps:#com[1]=='01' or com[1]=='07' or com[1]=='26' or
        # com[1]=='38'or com[1]=='42' or com[1]=='69' or com[1]=='73' or
        # com[1]=='74' :
        b[0] = com[4]
        b[1] = com[3]
        b = a@b  # multiply the matrix by the vector of the commune position

        strip = math.floor(b[1] / graphHeight) + NBSTRIPS // 2
        # if k==94001: print(repr(k)+"___"+repr(strip))
        if b[0] >= 0 and b[0] < graphWidth and strip >= 0 and strip < NBSTRIPS and len(com) > 5:

            segment = math.floor(b[0] / graphWidth * len(vals[strip][0]))
            # if k==94001: print(repr(k)+"___"+repr(strip)+" => "+repr(segment))

            for k in range(0, NBCANDS + 1):
                vals[strip][k][segment] += com[5 + k]

    # stop = timeit.default_timer()
    # print ("computeGraphs2 "+repr(stop - start))

    # normalize every point of the strip graphs
    for l in range(NBSTRIPS):

        # fill the empty data
        first = 0
        last = len(vals[l][0]) - 1
        while first < last and vals[l][0][first] == 0:
            first = first + 1
        while first < last and vals[l][0][last] == 0:
            last = last - 1
        for i in range(first, last - 1):
            if vals[l][0][i] == 0:
                ii = i
                while vals[l][0][ii] == 0:
                    ii = ii + 1
                for j in range(i + 1, ii - 1):
                    for k in range(0, NBCANDS + 1):
                        vals[l][k][j] = (vals[l][k][i]  * (ii - j) +
                                         vals[l][k][ii] * (j  - i)) / (ii - i)

        # smooth the data by bluring the signal
        for k in range(0, NBCANDS + 1):
            for i in range(1, len(vals[l][0]) - 1):
                vals[l][k + 6][i] = (vals[l][k][i - 1] + 2 * vals[l][k][i] + vals[l][k][i + 1]) / 4

            '''vals[l][7][i]  = (vals[l][2][i-1] + 2*vals[l][2][i] + vals[l][2][i+1])/4
      vals[l][8][i]  = (vals[l][3][i-1] + 2*vals[l][3][i] + vals[l][3][i+1])/4
      vals[l][9][i]  = (vals[l][4][i-1] + 2*vals[l][4][i] + vals[l][4][i+1])/4
      vals[l][10][i] = (vals[l][5][i-1] + 2*vals[l][5][i] + vals[l][5][i+1])/4
      vals[l][11][i] = (vals[l][0][i-1] + 2*vals[l][0][i] + vals[l][0][i+1])/4'''

        for i in range(0, len(vals[l][6])):
            v = vals[l][6][i]
            if v == 0:
                vals[l][0][i] = -1.0
                vals[l][1][i] = 0.0
                vals[l][2][i] = 0.0
                vals[l][3][i] = 0.0
                vals[l][4][i] = 0.0
                vals[l][5][i] = 0.0
            else:
                vals[l][0][i] = 0.0
                vals[l][1][i] = vals[l][7][i]  / v
                vals[l][2][i] = vals[l][8][i]  / v
                vals[l][3][i] = vals[l][9][i]  / v
                vals[l][4][i] = vals[l][10][i] / v
                vals[l][5][i] = vals[l][11][i] / v

    # stop = timeit.default_timer()
    # print ("computeGraphs3 "+repr(stop - start))

    for l in range(NBSTRIPS):
        ll = l - NBSTRIPS // 2
        prop = graphWidth / (len(vals[l][0]) - 1)
        for k in range(1, NBCANDS + 1):
            if vl[k - 1 + l * NBCANDS]:
                vvv = vl[k - 1 + l * NBCANDS].vertices

                for i in range(0, len(vals[l][0]) - 1):
                    v0 = vals[l][0][i + 0]
                    v1 = vals[l][0][i + 1]
                    v2 = vals[l][k][i + 0]
                    v3 = vals[l][k][i + 1]

                    if v0 < 0 or v1 < 0:
                        v1 = 0
                        v2 = 0
                        v2 = 0
                        v3 = 0
                    if v0 >= 0:
                        # cumulate y to make a stacked graph
                        vals[l][0][i + 0] += vals[l][k][i + 0]

                    x14 = (i + 0) * prop
                    y1  = (ll + v0 + v2) * graphHeight
                    x23 = (i + 1) * prop
                    y2  = (ll + v1 + v3) * graphHeight
                    y3  = (ll + v1) * graphHeight
                    y4  = (ll + v0) * graphHeight
                    vvv[i * 8 + 0] = x14
                    vvv[i * 8 + 1] = y1
                    vvv[i * 8 + 2] = x23
                    vvv[i * 8 + 3] = y2
                    vvv[i * 8 + 4] = x23
                    vvv[i * 8 + 5] = y3
                    vvv[i * 8 + 6] = x14
                    vvv[i * 8 + 7] = y4

                # cumulate last one since the previous for do not update the
                # last value
                vals[l][0][len(vals[0]) - 1] += vals[l][k][len(vals[l][0]) - 1]
            else:
                vertices = ()

                for i in range(0, len(vals[l][0]) - 1):
                    v0 = vals[l][0][i + 0]
                    v1 = vals[l][0][i + 1]
                    v2 = vals[l][k][i + 0]
                    v3 = vals[l][k][i + 1]
                    if v0 < 0 or v1 < 0:
                        v1 = 0
                        v2 = 0
                        v2 = 0
                        v3 = 0
                    if v0 >= 0:
                        # cumulate y to make a stacked graph
                        vals[l][0][i + 0] += vals[l][k][i + 0]

                    x14 = (i + 0) * prop
                    y1 = (ll + v0 + v2) * graphHeight
                    x23 = (i + 1) * prop
                    y2 = (ll + v1 + v3) * graphHeight
                    y3 = (ll + v1) * graphHeight
                    y4 = (ll + v0) * graphHeight

                    vertices += (x14, y1, x23, y2, x23, y3, x14, y4)

                vals[l][0][len(vals[0]) - 1] += vals[l][k][len(vals[l][0]) - 1]

                colors = allcolors[k - 1] * (len(vertices) // 2)
                if len(vertices) // 2 > 0:
                    vl[k - 1 + l * NBCANDS] = vertex_graphs.add(len(vertices) // 2, gl.GL_QUADS, None,
                                                          ('v2f', vertices),
                                                          ('c3B', colors)
                                                          )

    # stop = timeit.default_timer()
    # print ("computeGraphs4 "+repr(stop - start))


#*****************************************************************************80
#
# Return True if a coordinate (x, y) is inside a polygon defined by
#    a list of verticies [(x1, y1), (x2, x2), ... , (xN, yN)].
#
#  Discussion:
#    Reference: http://www.ariel.com.au/a/python-point-int-poly.html
##
#  Licensing:
#    ???
#
#  Modified:
#    11 July 2017
#
#  Author:
#    Fred Vernier
#
#  Parameters:
#    x,y : the position to test
#    points : a list of tuple/points (x,y) defining the polygon
#
def inside_polygon(x, y, points):
    n = len(points)
    inside = False
    p1x = points[0]
    p1y = points[1]
    for i in range(2, n - 1, 2):
        p2x = points[i % n]
        p2y = points[i % n+1]
        if y > min(p1y, p2y):
            if y <= max(p1y, p2y):
                if x <= max(p1x, p2x):
                    if p1y != p2y:
                        xinters = (y - p1y) * (p2x - p1x) / (p2y - p1y) + p1x
                    if p1x == p2x or x <= xinters:
                        inside = not inside
        p1x, p1y = p2x, p2y
    return inside


#*****************************************************************************80
#
# display the UI
#
#  Discussion:
#
#    pb is the progress bar. when negative displays the normal UI
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license.
#
#  Modified:
#
#    10 July 2017
#
#  Author:
#
#    Fred Vernier
#
#  Parameters:
#
#    dt : time since last update ?
#
def update(dt):
    global pb, maxpb, minlon, maxlon, minlat, maxlat, angle, zoom, window, depzoom, vals, legend, detcom, comdetail

    #print ('update '+repr(pb)+' / '+repr(maxpb))
    window.clear()
    tx = int(window.width/2)
    ty = int(window.height/2)

    gl.glEnable(gl.GL_BLEND)
    gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)

    # To display the UI, the trick is to setup the PROJECTION matrix first
    # centered
    gl.glMatrixMode(gl.GL_PROJECTION)
    gl.glLoadIdentity()
    gl.glOrtho(-tx, tx, -ty, ty, -1, 1)
    gl.glMatrixMode(gl.GL_MODELVIEW)

    if pb < 0:
        start = timeit.default_timer()
        # white bacckground
        pyglet.graphics.draw_indexed(4, gl.GL_TRIANGLES,
                                     [0, 1, 3, 0, 2, 3],
                                     ('v2i', (-tx, -ty,
                                              -tx,  ty,
                                              tx, -ty,
                                              tx,  ty)))

        # display polygons/departements
        gl.glPushMatrix()
        gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
        gl.glTranslatef(-mmx, -mmy, 0)
        a = (ctypes.c_float * 16)()
        mvm = gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
        a = np.array(a)
        a = np.reshape(a, (4, 4))
        a = a.T
        a = np.linalg.inv(a)
        gl.glPopMatrix()
        tl = np.array([-window.width/2, -window.height/2, 0, 1])
        tl = a@tl  # multiply the inverted matrix by the vector of the topleft
        br = np.array([+window.width/2, +window.height/2, 0, 1])
        br = a@br

        for k, dep in alldeps.items():
            gl.glPushMatrix()
            gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
            gl.glTranslatef(-mmx, -mmy, 0)
            gl.glTranslatef(dep['cenx'], dep['ceny'], 0)
            gl.glScalef(depzoom, depzoom, 1)
            gl.glTranslatef(-dep['cenx'], -dep['ceny'], 0)

            if tl[0] <= dep["bbox"][2] and br[0] >= dep["bbox"][0] and tl[1] <= dep["bbox"][3] and br[1] >= dep["bbox"][1]:
                dep['poly'].draw()

            gl.glPopMatrix()

        # display the strip-graphs
        gl.glPushMatrix()
        if NBSTRIPS == 1:
            gl.glTranslatef(x0, y0, 0)
        gl.glRotatef(angle, 0, 0, 1)
        gl.glTranslatef(-graphWidth/2, -graphHeight/2, 0)
        # for v in vertex_graphs:
        vertex_graphs.draw()
        # vertex_list2.draw(gl.GL_TRIANGLES)
        gl.glPopMatrix()

        # display borders
        gl.glPushMatrix()
        gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
        gl.glTranslatef(-mmx, -mmy, 0)
        a = (ctypes.c_float * 16)()
        mvm = gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
        a = np.array(a)
        a = np.reshape(a, (4, 4))
        a = a.T
        a = np.linalg.inv(a)
        tl = np.array([-window.width/2, -window.height/2, 0, 1])
        tl = a@tl  # multiply the inverted matrix by the vector of the topleft
        br = np.array([+window.width/2, +window.height/2, 0, 1])
        br = a@br
        gl.glPopMatrix()

        # draw departements borders and commune points
        gl.glPointSize(2)
        for k, dep in alldeps.items():
            gl.glPushMatrix()
            gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
            gl.glTranslatef(-mmx, -mmy, 0)

            if tl[0] <= dep["bbox"][2] and br[0] >= dep["bbox"][0] and tl[1] <= dep["bbox"][3] and br[1] >= dep["bbox"][1]:
                dep['border'].draw()
                if zoom > 1500:
                    dep['compoints'].draw()
            gl.glPopMatrix()

        # draw the commune points and point/name of the detail
        if comdetail:
            gl.glPushMatrix()
            gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
            gl.glTranslatef(-mmx, -mmy, 0)

            gl.glPointSize(5)
            pyglet.graphics.draw(
                1, gl.GL_POINTS, ('v2f', (comdetail[0][4], comdetail[0][3])))
            gl.glPointSize(2)
            for k, com in enumerate(comdetail):
                pyglet.graphics.draw(
                    1, gl.GL_POINTS, ('v2f', (comdetail[k][4], comdetail[k][3])))

            gl.glTranslatef(comdetail[0][4], comdetail[0][3], 0)
            gl.glScalef(1/(zoom*math.sin(math.pi/4)), 1/zoom, 1)
            gl.glTranslatef(-comdetail[0][4], -comdetail[0][3], 0)

            detcom.draw()
            gl.glPopMatrix()

        # display labels (names of departement)
        for k, dep in alldeps.items():
            gl.glPushMatrix()
            gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
            gl.glTranslatef(-mmx, -mmy, 0)
            gl.glTranslatef(dep['cenx'], dep['ceny'], 0)
            gl.glScalef(1/(zoom*math.sin(math.pi/4)), 1/zoom, 1)
            gl.glTranslatef(-dep['cenx'], -dep['ceny'], 0)
            dep['label'].draw()
            gl.glPopMatrix()

        stop = timeit.default_timer()
        #print ("update "+repr(stop - start))

    else:
        gl.glPushMatrix()
        gl.glTranslatef(-tx, -ty, 0)
        gl.glScalef(window.width/image1.width, window.height/image1.height, 1)
        image1.blit(0, 0)
        gl.glPopMatrix()

        pyglet.graphics.draw_indexed(6, gl.GL_TRIANGLES,
                                     [0, 1, 2, 1, 2, 3, 2, 3, 4, 3, 4, 5],
                                     ('v2f', (-tx+50, -60,
                                              tx-50,  -60,
                                              -tx+50, +20,
                                              tx-50,  +20,
                                              -tx+50, +60,
                                              tx-50,  +60)),
                                     ('c3B', (0,  96, 0, 0,  96, 0, 0,  64,
                                              0, 0,  64, 0, 0,  128, 0, 0,  128, 0))
                                     )
        pyglet.graphics.draw_indexed(6, gl.GL_TRIANGLES,
                                     [0, 1, 2, 1, 2, 3, 2, 3, 4, 3, 4, 5],
                                     ('v2f', (-tx+50, -60,
                                              -tx+50+pb *
                                              (window.width-100)/(maxpb), -60,
                                              -tx+50, +20,
                                              -tx+50+pb *
                                              (window.width-100)/(maxpb), +20,
                                              -tx+50, +60,
                                              -tx+50+pb*(window.width-100)/(maxpb), +60)),
                                     ('c3B', (0,  255, 0, 0,  255, 0,    128,  255, 128,
                                              128,  255, 128,     64,  255, 64, 64,  255, 64))
                                     )
        msg.draw()

    # and 4 little logos
    logo1.blit(tx-logo1.width,                         ty-logo1.height)
    logo2.blit(-tx,                                    ty-logo2.height)
    logo3.blit(tx-logo1.width-logo4.width-logo3.width, ty-logo3.height)
    logo4.blit(tx-logo1.width-logo4.width,             ty-logo4.height)

    legend.draw()


def update2(dt):
    update(dt)
    window.flip()


@window.event
def on_draw():
    update(-1)


#pyglet.clock.schedule_interval(update2, 0.1)
# computeImages()
computePolygons()
loadData()
computeGraphs()


@window.event
def on_key_press(symbol, modifiers):
    global glomodifier, zoom, mmx, mmy, NBSTRIPS, vl, vertex_graphs
    if symbol == key.Q:
        print('The "Q" key was pressed.')
        pyglet.app.exit()
        return pyglet.event.EVENT_HANDLED
    elif symbol == 65505:
        glomodifier = 1
    elif symbol == 65517 or symbol == 65519:
        glomodifier = 2
    elif symbol == key.X:
        print('The "X" key was pressed.')
        zoom = window.height/(maxlat-minlat)
        mmx = (minlon+maxlon)/2
        mmy = (minlat+maxlat)/2
        computeGraphs()
    elif symbol == key.S:
        print('The "S" key was pressed.')
        if NBSTRIPS == 1:
            NBSTRIPS = NBMAXSTRIPS
            for v in vl:
                v.delete()
            vertex_graphs = pyglet.graphics.Batch()
            vl = [None] * (NBCANDS*NBSTRIPS)
        else:
            NBSTRIPS = 1
            for v in vl:
                v.delete()
            vertex_graphs = pyglet.graphics.Batch()
            vl = [None] * (NBCANDS*NBSTRIPS)
        computeGraphs()
    else:
        print("key pressed "+repr(symbol))


@window.event
def on_key_release(symbol, modifiers):
    global glomodifier
    if symbol == 65505 or symbol == 65517 or symbol == 65519:
        glomodifier = 0


#*****************************************************************************80
#
# callback when the mouse button is pressed
#
#  Discussion:
#
#    first retrieve the transfotmation matrix, tweak it and inverse it
#    to then transform the mouse position with it. It gives the mouse position
#    in the map coordinates
#
#  Licensing:
#
#    This code is distributed under the GNU LGPL license.
#
#  Modified:
#
#    10 July 2017
#
#  Author:
#
#    Fred Vernier
#
#  Parameters:
#
#    dt : x,y the position of the mouse
#
@window.event
def on_mouse_press(x, y, button, modifiers):
    global x0, y0, zoom, mmx, mmy, mmx0, mmy0, comdetail, communes, detcom
    if button == mouse.RIGHT:
        print('The right mouse button was pressed at '+repr(x)+','+repr(y))
    else:
        gl.glPushMatrix()
        gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
        gl.glTranslatef(-mmx, -mmy, 0)

        a = (ctypes.c_float * 16)()
        mvm = gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
        a = np.array(a)
        a = np.reshape(a, (4, 4))
        gl.glPopMatrix()
        a = a.T
        a = np.linalg.inv(a)
        b = np.array([x-window.width/2, y-window.height/2, 0, 1])
        b = a@b  # multiply the inverted matrix by the vector of the mous position
        #print (b)
        mmx0 = b[0]
        mmy0 = b[1]

        gl.glPushMatrix()
        gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
        gl.glTranslatef(-mmx, -mmy, 0)
        a = (ctypes.c_float * 16)()
        mvm = gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
        a = np.array(a)
        a = np.reshape(a, (4, 4))
        a = a.T
        gl.glPopMatrix()

        mind = 10000
        closecom = None
        comdetail = []
        b = np.array([0, 0, 0, 1])

        for k, dep in alldeps.items():
            if (mmx0 >= dep["bbox"][0] and mmx0 <= dep["bbox"][2] and mmy0 >= dep["bbox"][1] and mmy0 <= dep["bbox"][3]):
                isin = False
                for points in dep["pieces"]:
                    isin = isin or inside_polygon(mmx0, mmy0, points)
                if isin:
                    print("MP IN "+dep["name"])
                    for i in range(0, len(dep['vert']), 2):
                        b[0] = dep['vert'][i]
                        b[1] = dep['vert'][i+1]
                        b = a@b  # multiply the matrix by the vector of commune position
                        d = math.hypot(x-window.width/2 -
                                       b[0], y-window.height/2-b[1])
                        if closecom == None or d < mind:
                            closecom = dep['ins'][i//2]
                            mind = d
                        #print (repr(dep['vert'][i])+","+repr(dep['vert'][i+1])+" => "+communes[dep['ins'][i//2]][2])

    if closecom:
        # update detail of the closest commune
        comdetail.append(communes[closecom])
        docud.text  = communes[closecom][2]+"\n  MELANCHON:"+"{:2.2f}".format(100*communes[closecom][10]/communes[closecom][5])+"%" +\
                                            "\n  HAMON:"+"{:2.2f}".format(100*communes[closecom][9]/communes[closecom][5])+"%" +\
                                            "\n  MACRON:"+"{:2.2f}".format(100*communes[closecom][8]/communes[closecom][5])+"%" +\
                                            "\n  FILLON:"+"{:2.2f}".format(100*communes[closecom][7]/communes[closecom][5])+"%" +\
                                            "\n  LE PEN:" + \
            "{:2.2f}".format(100*communes[closecom]
                             [6]/communes[closecom][5])+"%"
        detcom.x = communes[closecom][4]
        detcom.y = communes[closecom][3]

        # find the strip and bin of the detail commune
        gl.glPushMatrix()
        gl.glTranslatef(graphWidth/2, graphHeight/2, 0)
        gl.glRotatef(-angle, 0, 0, 1)
        if NBSTRIPS == 1:
            gl.glTranslatef(-x0, -y0, 0)
        gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
        gl.glTranslatef(-mmx, -mmy, 0)
        a = (ctypes.c_float * 16)()
        mvm = gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
        a = np.array(a)
        a = np.reshape(a, (4, 4))
        a = a.T
        gl.glPopMatrix()

        b = np.array([communes[closecom][4], communes[closecom][3], 0, 1])
        b = a@b  # multiply the matrix by the vector of commune position
        strip0 = math.floor(b[1]/graphHeight)+NBSTRIPS//2
        segment0 = math.floor(b[0]/graphWidth*len(vals[strip0][0]))

        for k, com in communes.items():
            # com[1]=='01' or com[1]=='07' or com[1]=='26' or com[1]=='38'or
            # com[1]=='42' or com[1]=='69' or com[1]=='73' or com[1]=='74' :
            if com[1] in alldeps:
                b[0] = com[4]
                b[1] = com[3]
                b = a@b  # multiply the matrix by the vector of commune position

                strip = math.floor(b[1]/graphHeight)+NBSTRIPS//2
                #if k==26001: print(repr(k)+"___"+repr(strip))
                if b[0] >= 0 and b[0] < graphWidth and strip >= 0 and strip < NBSTRIPS and len(com) > 5:

                    segment = math.floor(b[0]/graphWidth*len(vals[strip][0]))
                    #if k==26001: print(repr(k)+"___"+repr(strip)+" => "+repr(segment))

                    if segment == segment0 and strip == strip0:
                        comdetail.append(com)


@window.event
def on_mouse_drag(x, y, dx, dy, button, modifiers):
    global zoom, mmx, mmy, mmx0, mmy0, x0, y0
    x0 = min(window.width,  max(0, x))-window.width/2
    y0 = min(window.height, max(0, y))-window.height/2

    if button == mouse.LEFT:
        gl.glPushMatrix()
        gl.glScalef(zoom*math.sin(math.pi/4), zoom, 1)
        gl.glTranslatef(-mmx, -mmy, 0)

        a = (ctypes.c_float * 16)()
        mvm = gl.glGetFloatv(gl.GL_MODELVIEW_MATRIX, a)
        a = np.array(a)
        a = np.reshape(a, (4, 4))
        a = a.T
        a = np.linalg.inv(a)
        b = np.array([x-window.width/2, y-window.height/2, 0, 1])
        b = a@b  # multiply the inverted matrix by the vector of the mous position
        mmx -= b[0]-mmx0
        mmy -= b[1]-mmy0
        gl.glPopMatrix()
        computeGraphs()


@window.event
def on_mouse_release(x, y, button, modifiers):
    pass


@window.event
def on_mouse_motion(x, y, dx, dy):
    global x0, y0, window
    x0 = min(window.width,  max(0, x))-window.width/2
    y0 = min(window.height, max(0, y))-window.height/2

    if NBSTRIPS == 1:
        computeGraphs()


@window.event
def on_mouse_scroll(x, y, scroll_x, scroll_y):
    global angle, zoom, depzoom, glomodifier

    # print(glomodifier)
    if glomodifier == 1:
        depzoom = min(1, max(0.1, depzoom+scroll_x/30.0))
    elif glomodifier == 2:
        angle -= scroll_y/10.0
    else:
        zoom -= scroll_y/1.0
        # print(zoom)
    computeGraphs()


@window.event
def on_window_close(window):
    event_loop.exit()
    return pyglet.event.EVENT_HANDLED


pyglet.app.run()
